import React, { Component } from 'react';
import PropTypes from 'prop-types';
import Paper from '../Paper/Paper';
import AppBarRightToolbar from './AppBarRightToolbar';
import Dom from '../utils/Dom';
import Events from '../utils/Events';
import { styles } from './styles';
import injectStyles from '../styles/injectStyles';

/**
 * AppBar - the main toolbar for the application
 */
class AppBar extends Component {
  /**
   * Set accepted properties
   */
  static propTypes = {
    /**
     * Custom classname
     */
    className: PropTypes.string,

    /**
     * JSS classes for styling the component
     */
    classes: PropTypes.object,

    /**
     * Icon element to show on the left
     */
    iconElementLeft: PropTypes.element,

    /**
     * Icon element or elements to show on the right
     */
    iconElementRight: PropTypes.oneOfType([PropTypes.element, PropTypes.array]),

    /**
     * The main title of the AppBar
     */
    title: PropTypes.node,

    /**
     * Optional custom z-depth
     */
    zDepth: PropTypes.number,

    /**
     * AppBaR will always remain at the top of the page on scroll
     *
     * Fixed will override this option if both are true
     */
    alwaysVisible: PropTypes.bool,

    /**
     * AppBaR is in a fixed location
     */
    fixed: PropTypes.bool,

    /**
     * Optional. Set this flag if we want to resize the toolbar on window.load() event.
     */
    resizeOnWindowLoad: PropTypes.bool,
  };

  /**
   * Set property defaults
   */
  static defaultProps = {
    title: '',
    zDepth: 1,
    alwaysVisible: false,
    fixed: false,
  };

  /**
   * Class constructor
   *
   * @param {Object} props Properties to send to the render function
   */
  constructor(props) {
    // Call paprent constructor
    super(props);

    // Variables used for callback refs
    this.outerCon = null;
    this.innerCon = null;

    this.state = {
      startTopOffset: 0,
      startWidth: 'auto',
      curTopOffset: -1,
    };
  }

  /**
   * Perform pop-dom-mount operations like listening for scroll and resize
   */
  componentDidMount() {
    // Set the window.load() event if specified in the props.
    if (this.props.resizeOnWindowLoad) {
      Events.on(window, 'load', this._onResize);
    }

    // Save the original top position of the menu
    if (this.outerCon) {
      this._onResize();
    }

    Events.on(window, 'scroll', this._onWindowScroll);
    Events.on(window, 'resize', this._onResize);
  }

  /**
   * Unregister all window listeners
   */
  componentWillUnmount() {
    Events.off(window, 'scroll', this._onWindowScroll);
    Events.off(window, 'resize', this._onResize);

    if (this.props.resizeOnWindowLoad) {
      Events.off(window, 'load', this._onResize);
    }
  }

  /**
   * Redner the component
   */
  render() {
    let classes = this.props.classes.appBar,
      elementCenter,
      menuElementLeft,
      menuElementRight;

    // Add color scheme
    classes += ' ' + this.props.classes.primary;

    if (this.props.className) {
      classes += ' ' + this.props.className;
    }

    // Set the left elements
    if (this.props.iconElementLeft) {
      // Add any appBar custom styles to the icon
      let iconElementLeft = this.props.iconElementLeft;
      menuElementLeft = (
        <div className={this.props.classes.innerLeftContainer}>{iconElementLeft}</div>
      );
    }

    // If right elements exists, wrap in a toolbar
    if (this.props.iconElementRight) {
      menuElementRight = <AppBarRightToolbar>{this.props.iconElementRight}</AppBarRightToolbar>;
    }

    // Add main content which is normally the title
    if (this.props.title) {
      // If the title is a string, wrap in an h1 tag.
      // If not, just use it as a node.
      elementCenter =
        toString.call(this.props.title) === '[object String]' ? (
          <div className={this.props.classes.innerCenterContainer}>{this.props.title}</div>
        ) : (
          this.props.title
        );
    } else if (this.props.children) {
      elementCenter = (
        <div className={this.props.classes.innerCenterContainer}>{this.props.children}</div>
      );
    }

    // Get the zDepth passed - we may increment if we are floating
    let zDepth = this.props.zDepth;

    // Handle offset when the document scrolls and the appbar is fixed
    let innerConStyle = null;
    if (this.props.fixed) {
      innerConStyle = {
        position: 'fixed',
        width: this.state.startWidth,
        zIndex: 5,
      };
    } else if (this.props.alwaysVisible && this.state.curTopOffset !== -1) {
      innerConStyle = {
        top: this.state.curTopOffset + 'px',
        width: this.state.startWidth,
        position: 'fixed',
        zIndex: 5,
      };
    }

    return (
      <div
        ref={outerCon => {
          this.outerCon = outerCon;
        }}
        className={this.props.classes.outerContainer}
      >
        <div style={innerConStyle}>
          <Paper rounded={false} className={classes} zDepth={zDepth}>
            {menuElementLeft}
            {elementCenter}
            {menuElementRight}
          </Paper>
        </div>
      </div>
    );
  }

  /**
   * Handle when the document is scrolled while the
   * The starting top of this menu was not 0 so it means
   * the menu is a fixed position and docked. A menu can be docked
   * below the top of the page (like below an AppBar) so we
   * want to be able to reposition the leftnav when the user scrolls
   * so it scrolls with the document until 0 (top)
   */
  _onWindowScroll = () => {
    // If the starting state was 0 then do nothing
    if (!this.props.alwaysVisible) {
      return;
    }

    // Get the scroll offset of the window
    let windowOffset = Dom.scrollOffset();

    /*
     * If we have scrolled, then follow the scroll.
     * Because the left nav div is position:fixed, then we
     * can move all the way to 0 to be at the top no matter how
     * far down the page they scroll
     */
    let newTop = this.state.startTopOffset - windowOffset.top;
    if (newTop < 0) {
      newTop = 0;
    }

    // Restore the original state if we are back in the viewport.
    if (windowOffset.top <= this.state.startTopOffset) {
      newTop = -1; // Reset
    }

    // Set state
    this.setState({ curTopOffset: newTop });
  };

  /**
   * Handle resize events
   */
  _onResize = () => {
    // If we never have a fixed element then do nothing
    if (!this.outerCon || (!this.props.alwaysVisible && !this.props.fixed)) {
      return;
    }

    let offset = Dom.offset(this.outerCon);
    this.setState({
      startTopOffset: offset.top,
      startWidth: offset.width + 'px',
    });
  };
}

// Wrap in JSS for styles
//export default injectSheet(styles)(AppBar);
export default injectStyles(styles, AppBar);
