import React, { Component } from 'react';
import ReactDOM from 'react-dom';
import Dom from '../utils/Dom';
import PropTypes from 'prop-types';
import AppBarSelectButton from './SelectButton';
import AppBarIconButton from './IconButton';
import injectStyles from '../styles/injectStyles';
import { styles } from './styles';
import ExpandMoreIcon from '../icons/svg/ExpandMoreIcon';
import UAParser from 'ua-parser-js';

const uaParser = new UAParser();

/**
 * The right toolbar for an AppBar
 *
 * @param props
 * @param context
 * @returns {ReactDOM}
 * @constructor
 */
class AppBarRightToolbar extends Component {
  /**
   * Set accepted properties
   */
  static propTypes = {
    /**
     * Optional class override
     */
    className: PropTypes.string,

    /**
     * JSS classes for styling the component
     */
    classes: PropTypes.object,

    /**
     * Child elements (text and icon)
     */
    children: PropTypes.node,
  };

  /**
   * Set property defaults
   */
  static defaultProps = {
    className: null,
  };

  /**
   * Class constructor
   *
   * @param {Object} props Properties to send to the render function
   */
  constructor(props) {
    // Call paprent constructor
    super(props);

    // Variables used for callback refs
    this.appBarRightContainer = null;
    this.appBarInnerContainer = null;

    this.state = {
      useSelectButton: false,
    };
  }

  componentDidMount() {
    this.shouldRightIconsMoveToSelectButton();
  }

  componentDidUpdate() {
    this.shouldRightIconsMoveToSelectButton();
  }

  render() {
    let classes = this.props.classes.innerRightContainer;
    if (this.props.className) {
      classes += ' ' + this.props.className;
    }

    let selectButtonIcons = [];
    let primaryIconButton = null;
    let minIconCountForSelectButton = 1;
    let iconButtons = [];
    let iconsForSelectButton = [];

    if (Array.isArray(this.props.children) == false) {
      // If there is only one icon then add it
      iconButtons.push(this.props.children);
    } else {
      // If we have more than one entry as children then we may need to fold them into a 'more' dropdown
      for (let idx in this.props.children) {
        let currentIcon = this.props.children[idx];

        // Let's verify if the currentIcon is a valid react element
        if (!currentIcon || !currentIcon.props) {
          continue;
        }

        // If we are dealing with primary icon and we dont have one, then we need to display it separately
        const isPrimaryIcon = currentIcon.props.isPrimary || false;

        if (isPrimaryIcon && !primaryIconButton) {
          minIconCountForSelectButton -= 1;
          primaryIconButton = currentIcon;
        } else if (
          currentIcon.type.name === 'AppBarSelectButton' ||
          currentIcon.type.name === 'SelectButton'
        ) {
          // If we are dealing with dropdown icons, then we need to display it separately too
          minIconCountForSelectButton -= 1;
          selectButtonIcons.push(currentIcon);
        } else {
          iconButtons.push(currentIcon);

          const menuIcon = React.cloneElement(currentIcon, {
            key: 'menuIcon' + idx,
            primary: true,
          });

          iconsForSelectButton.push(menuIcon);
        }
      }

      // We only need to use the select button if we are dealing with 2 or more icon buttons
      if (this.state.useSelectButton && iconButtons.length > minIconCountForSelectButton) {
        const menuItems = [];
        iconButtons = (
          <AppBarSelectButton menuItems={menuItems} icon={<ExpandMoreIcon />}>
            {iconsForSelectButton}
          </AppBarSelectButton>
        );
      }
    }

    return (
      <div
        className={classes}
        ref={appBarRightContainer => {
          this.appBarRightContainer = appBarRightContainer;
        }}
      >
        {primaryIconButton}
        {iconButtons}
        {selectButtonIcons}
      </div>
    );
  }

  /**
   * This will check if current toolbar right icons are too many to be displayed inline
   * If the right icons will overflow, we will set the state.useSelectButton to true
   * The right icons will be moved inside the select button dropdown
   */
  shouldRightIconsMoveToSelectButton = () => {

    /*
     * If we have already evaluated that the right icons will be displayed in the select dropdown
     * Then, there is no need to re-evaluate
     */
    if (this.state.useSelectButton) {
      return;
    }

    const domAppBarRight = ReactDOM.findDOMNode(this.appBarRightContainer);
    const domAppBarParent =
      domAppBarRight && domAppBarRight.parentNode ? domAppBarRight.parentNode : null;

    if (domAppBarRight && domAppBarParent) {
      const offset = Dom.offset(domAppBarRight);
      const parentOffset = Dom.offset(domAppBarParent);

      /*
       * Check if appbarRight height is greater than the appbar client height
       * If the toolbar right icons is too many, the appBarRightContainer will overflow and its height will increase
       */
      if (offset.height > parentOffset.height) {
        // If so, then we need to set the state so it will use the SelectButton to display the icons
        this.setState({ useSelectButton: true });
      }
    }
  }
}

// Wrap in JSS for styles
export default injectStyles(styles, AppBarRightToolbar);
