import React from 'react';
import PropTypes from 'prop-types';
import TouchRipple from '../ripples/TouchRipple';
import FocusRipple from '../ripples/FocusRipple';
import classnames from 'classnames';
import { styles } from './styles';
import injectStyles from '../styles/injectStyles';

/**
 * Functional component for any button
 *
 * @param props
 * @param context
 * @returns {ReactDOM}
 * @constructor
 */
const Button = (props, context) => {
  // Setup classes
  const type = props.type;
  const propsClasses = props.classes;
  const classes = classnames(
    propsClasses[type],
    {
      [propsClasses[type + 'Primary']]: props.primary,
      [propsClasses[type + 'Accent']]: props.accent,
      [propsClasses[type + 'Disabled']]: props.disabled,
      [propsClasses[type + 'Depressed']]: props.depressed,
      [propsClasses[type + 'Mini']]: props.mini,
    },
    props.className,
  );

  // Determine if the selected type of button is a centered ripple
  let centerRipple = type == 'floating' || type == 'icon';
  const label = props.label ? props.label : props.children;

  const onClickHandler = props.onTap || props.onClick;

  if (onClickHandler && !props.disabled) {
    return (
      <button className={classes} onClick={onClickHandler}>
        <FocusRipple />
        <TouchRipple centerRipple={centerRipple}>{label}</TouchRipple>
      </button>
    );
  } else {
    return (
      <button disabled={props.disabled} className={classes}>
        {label}
      </button>
    );
  }
};

/**
 * Set accepted properties
 */
Button.propTypes = {
  /**
   * Secondary accent color
   */
  accent: PropTypes.bool,

  /**
   * Primary button color and behavior
   */
  primary: PropTypes.bool,

  /**
   * Child elements (text and icon)
   */
  children: PropTypes.node,

  /**
   * Optional classname override
   */
  className: PropTypes.string,

  /**
   * Disabled flag - cannot be interacted with
   */
  disabled: PropTypes.bool,

  /**
   * Flat style (looks like a link almost)
   */
  flat: PropTypes.bool,

  /**
   * Floating button like floating action buttons (FAB) in material
   */
  floating: PropTypes.bool,

  /**
   * Link reference to go to
   */
  href: PropTypes.string,

  /**
   * ?
   */
  inverse: PropTypes.bool,

  /**
   * Optional icon to display to the left of the text
   */
  icon: PropTypes.oneOfType([PropTypes.string, PropTypes.element]),

  /**
   * Optional property in place of child text
   */
  label: PropTypes.string,

  /**
   * Small button
   */
  mini: PropTypes.bool,

  /**
   * If true the button should appear pressed or down (think toggle)
   */
  depressed: PropTypes.bool,
  neutral: PropTypes.bool,
  onMouseLeave: PropTypes.func,
  onMouseUp: PropTypes.func,

  /**
   * Event triggered when the user taps/clicks on the button
   */
  onTape: PropTypes.func,

  /**
   * Alias for onTap for backwards compatibility
   */
  onClick: PropTypes.func,

  raised: PropTypes.bool,
  type: PropTypes.oneOf(['raised', 'flat', 'floating', 'icon']),
};

/**
 * Set property defaults
 */
Button.defaultProps = {
  accent: false,
  className: '',
  flat: false,
  floating: false,
  mini: false,
  neutral: true,
  primary: false,
  raised: false,
  type: 'raised',
};

// Wrap in JSS for styles
export default injectStyles(styles, Button);
