import React, { Component } from 'react';
import PropTypes from 'prop-types';
import KeyCode from '../utils/KeyCode';
import FlatButton from '../Button/FlatButton';
import Overlay from '../Overlay/Overlay';
import Paper from '../Paper/Paper';
import { styles } from './styles';
import injectStyles from '../styles/injectStyles';

/**
 * Dialog window component
 */
class Dialog extends Component {
  static propTypes = {
    /**
     * Array of possible actions to create buttons for
     */
    actions: PropTypes.array,

    /**
     * Which action should get focus when loaded
     */
    actionFocus: PropTypes.string,

    /**
     * Automatically detect the ideal height of the window
     */
    autoDetectWindowHeight: PropTypes.bool,

    /**
     * Automatically scroll the body of the window if larger than viewable area
     */
    autoScrollBodyContent: PropTypes.bool,

    /**
     * Class to theme the content container
     */
    contentClassName: PropTypes.string,

    /**
     * Depricated. Should be using showDialog.
     *
     * If true, open as soon as we mount into the dom.
     */
    openImmediately: PropTypes.bool,

    /**
     * Action to perform when a user clicks away from the dialog
     */
    onClickAway: PropTypes.func,

    /**
     * Callback to call when the dialog is dismissed
     */
    onDismiss: PropTypes.func,

    /**
     * Callback to call when the dialog is shown
     */
    onShow: PropTypes.func,

    /**
     * Automatically reposition the dialog if the contents change
     */
    repositionOnUpdate: PropTypes.bool,

    /**
     * Do not allow the user to dismiss by clicking off the dialog
     */
    modal: PropTypes.bool,

    /**
     * The title for the dialog window
     */
    title: PropTypes.node,

    /**
     * Flag to set this dialog to take up 100% of the page
     */
    fullPage: PropTypes.bool,

    /**
     * Flag that will determine if we are going to show the dialog or not
     */
    showDialog: PropTypes.bool,
  };

  static defaultProps = {
    autoDetectWindowHeight: false,
    autoScrollBodyContent: false,
    actions: [],
    repositionOnUpdate: true,
    modal: false,
    openImmediately: false,
  };

  /**
   * Class constructor
   *
   * @param {Object} props Properties to send to the render function
   */
  constructor(props) {
    // Call parent constructor
    super(props);

    // Create some references to dom elements for sizing
    this.refDialogWindow = React.createRef();
    this.refDialogBody = React.createRef();
    this.refDialogActions = React.createRef();

    this.state = {
      // Used for dyanmic height (and scroll) of body for long content
      dialogBodyMaxHeight: null,
      // Used for auto positioning to the middle of the screen
      dialogWindowMarginTop: null,
    };
  }

  componentDidMount() {
    // Variables used for callback refs
    this.dialogOverlay = null;
    this._positionDialog();

    // TODO
    // 'keyup': '_handleWindowKeyUp'

    if (this.props.showDialog && this.dialogOverlay) {
      this.dialogOverlay.preventScrolling();
      this._onShow();
      this._focusOnAction();
    }
  }

  componentDidUpdate(prevProps, prevState) {
    // Ignore state change caused by _positionDialog() to avoid a continual loop
    if (
      this.state.dialogBodyMaxHeight == prevState.dialogBodyMaxHeight &&
      this.state.dialogWindowMarginTop == prevState.dialogWindowMarginTop
    ) {
      this._positionDialog();
      this._focusOnAction();
    }
  }

  render() {
    const propsClasses = this.props.classes;

    // Set classes for window contents
    let classesWindow = propsClasses.dialogWindow;
    if (this.props.windowClassName) {
      classesWindow += ' ' + this.props.windowClassName;
    }

    // Set classes for dialog window
    let classesDialog = propsClasses.dialog;
    if (this.props.showDialog) {
      classesDialog += ' ' + propsClasses.dialogIsShown;

      if (this.props.fullPage) {
        classesWindow += ' ' + propsClasses.dialogWindowFullPage;
      } else {
        classesWindow += ' ' + propsClasses.dialogWindowInside;
      }
    }

    if (this.props.className) {
      classesDialog += ' ' + this.props.className;
    }

    // Set classes for window contents
    let classesWindowBody = propsClasses.dialogWindowBody;
    if (this.props.dialogWindowBodyClassName) {
      classesWindowBody += ' ' + this.props.dialogWindowBodyClassName;
    }

    // Add title
    let title;
    if (this.props.title) {
      // If the title is a string, wrap in an h3 tag.
      // If not, just use it as a node.
      title =
        Object.prototype.toString.call(this.props.title) === '[object String]' ? (
          <h3 className={propsClasses.dialogWindowTitle}>{this.props.title}</h3>
        ) : (
          this.props.title
        );
    }

    // Get actions to display at the bottom
    const actions = this._getActionsContainer(this.props.actions);

    return (
      <div className={classesDialog}>
        <div
          className={classesWindow}
          ref={this.refDialogWindow}
          style={
            this.state.dialogWindowMarginTop ? { marginTop: this.state.dialogWindowMarginTop } : {}
          }
        >
          <Paper zDepth={4}>
            {title}
            <div
              ref={this.refDialogBody}
              style={
                this.state.dialogBodyMaxHeight ? { maxHeight: this.state.dialogBodyMaxHeight } : {}
              }
              className={classesWindowBody}
            >
              {this.props.children}
            </div>
            <div ref={this.refDialogActions}>{actions}</div>
          </Paper>
        </div>
        <Overlay
          ref={dialogOverlay => {
            this.dialogOverlay = dialogOverlay;
          }}
          show={this.props.showDialog}
          onClick={this._handleOverlayTouchTap}
        />
      </div>
    );
  }

  isOpen() {
    return this.props.showDialog;
  }

  dismiss = () => {
    this._onDismiss();
  };

  show = () => {
    this._focusOnAction();
    this._onShow();
  };

  /**
   * Reposition the dialog - usually means dynamic content changed
   */
  reposition() {
    this._positionDialog();
  }

  _addClassName(reactObject, className) {
    const originalClassName = reactObject.props.className;
    const newClassname = originalClassName ? originalClassName + ' ' + className : className;

    return React.cloneElement(reactObject, { className: newClassname });
  }

  _getAction(actionJSON, key) {
    const props = {
      key: key,
      secondary: true,
      onClick: actionJSON.onClick ? actionJSON.onClick : this.dismiss,
      label: actionJSON.text,
    };
    if (actionJSON.ref) {
      props.ref = actionJSON.ref;
      props.keyboardFocused = actionJSON.ref === this.props.actionFocus;
    }

    return <FlatButton {...props} />;
  }

  _getActionsContainer(actions) {
    let actionContainer;
    let actionObjects = [];

    if (actions.length) {
      for (let i = 0; i < actions.length; i++) {
        let currentAction = actions[i];

        //if the current action isn't a react object, create one
        if (!React.isValidElement(currentAction)) {
          currentAction = this._getAction(currentAction, i);
        }

        currentAction = this._addClassName(currentAction, this.props.classes.dialogWindowAction);
        actionObjects.push(currentAction);
      }

      actionContainer = (
        <div className={this.props.classes.dialogWindowActions}>{actionObjects}</div>
      );
    }

    return actionContainer;
  }

  /**
   * Make sure the dialog is correct positioned on the screen
   */
  _positionDialog() {
    if (this.props.showDialog && window) {
      const clientHeight = window.innerHeight;
      let dialogWindow = this.refDialogWindow.current;
      let dialogBody = this.refDialogBody.current;
      const minMarginTop = 0;

      // If we are running server-side or in automated tests element is not mounted
      if (!clientHeight || !dialogWindow || !dialogBody) {
        return;
      }

      // Props to set that will be used in this.setState at the end
      let dialogBodyMaxHeight = null;
      let dialogWindowMarginTop = null;

      const dialogWindowHeight = dialogWindow.offsetHeight;
      let windowTop = (clientHeight - dialogWindowHeight) / 2 - 2 * 64;

      // Set to full-height if we are in auto-detect
      if (this.props.autoDetectWindowHeight || this.props.autoScrollBodyContent) {
        windowTop = 0;
      } else if (windowTop < minMarginTop) {
        windowTop = minMarginTop;
      }

      //Vertically center the dialog window, but make sure it doesn't
      //transition to that position.
      if (this.props.repositionOnUpdate || !dialogWindow.style.marginTop) {
        dialogWindowMarginTop = windowTop + 'px';
      }

      // Force a height if the dialog is taller than clientHeight
      if (this.props.autoDetectWindowHeight || this.props.autoScrollBodyContent) {
        let maxDialogContentHeight = clientHeight - 2 * 64;

        if (this.props.title) {
          maxDialogContentHeight -= dialogBody.previousSibling.offsetHeight;
        }

        if (this.props.actions.length) {
          maxDialogContentHeight -= dialogBody.nextSibling.offsetHeight;
        }

        dialogBodyMaxHeight = maxDialogContentHeight + 'px';
      }

      // setState only if values changed (reduce re-rendering unnecessarily)
      if (
        this.state.dialogBodyMaxHeight != dialogBodyMaxHeight ||
        this.state.dialogWindowMarginTop != dialogWindowMarginTop
      ) {
        this.setState({
          dialogBodyMaxHeight: dialogBodyMaxHeight,
          dialogWindowMarginTop: dialogWindowMarginTop,
        });
      }
    }
  }

  _focusOnAction() {
    if (this.props.actionFocus) {
      console.error('We need to find a better way to do this');
      // if (ReactDOM.findDOMNode(this.refs[this.props.actionFocus])) {
      //   ReactDOM.findDOMNode(this.refs[this.props.actionFocus]).focus();
      // }
    }
  }

  _onShow() {
    if (this.props.onShow) this.props.onShow();
  }

  _onDismiss() {
    if (this.props.onDismiss) this.props.onDismiss();
  }

  _handleOverlayTouchTap = e => {
    if (!this.props.modal) {
      this.dismiss();
      if (this.props.onClickAway) this.props.onClickAway();
    }
  };

  _handleWindowKeyUp = e => {
    if (!this.props.modal && e.keyCode == KeyCode.ESC) {
      this.dismiss();
    }
  };
}

// Wrap in JSS for styles
export default injectStyles(styles, Dialog);
