import React, { Component } from 'react';
import PropTypes from 'prop-types';
import RichText from '../Input/RichText';
import RichTextIframe from '../Input/RichTextIframe';
import EditorToolbar from './EditorToolbar';
import { styles } from './styles';
import injectStyles from '../styles/injectStyles';
import UAParser from 'ua-parser-js';

/**
 * Contains both the toolbar and an instance of RichText component
 */
class ContentHtml extends Component {
  /**
   * Class constructor
   *
   * @param {Object} props Properties to send to the render function
   */
  constructor(props) {
    // Call parent constructor
    super(props);

    this.state = {
      value: this.props.value,
      toggleStyle: null,
      toggleType: null,
      // Used to change behavior based on problems with android
      deviceOsName: null,
      deviceBrowserName: null,
    };
  }

  /**
   * Handle when the editor has changed
   *
   * @param {string} value The value of the editor
   * @private
   */
  _onChange = value => {
    if (this.props.onChange) {
      this.props.onChange(value);
    }

    this.setState({ value });
  };

  _onToggle = value => {
    if (this.props.onChange) {
      this.props.onChange(value);
    }

    this.setState({
      value,
      toggleStyle: null,
      toggleType: null,
    });
  };

  /**
   * Handle when the editor looses the focus
   *
   * @param {string} value The value of the editor
   * @private
   */
  _onBlur = value => {
    if (this.props.onBlur) {
      this.props.onBlur(value);
    } else {
      //this.setState({value});
    }
  };

  /**
   * Handles when the user set the focus in the editor
   *
   * @param {string} value The value of the editor
   * @private
   */
  _onFocus = value => {
    if (this.props.onFocus) {
      this.props.onFocus(value);
    } else {
      //this.setState({value});
    }
  };

  /**
   * Handles the toggling of styles in the toolbar icons
   *
   * @param {string} style The style that was clicked. It is either block style or inline style
   * @param {string} type The style type that was toggled. (e.g. bold, italic, underline, h1, h2, h3, ul, ol)
   * @private
   */
  _handleStyleToggle = (style, type) => {
    this.setState({
      toggleStyle: style,
      toggleType: type,
    });
  };

  /**
   * Handles the toggling of content view
   *
   * @param {int} contentView The content view we are switching to
   * @private
   */
  _handleContentViewToggle = contentView => {
    if (this.props.onContentViewToggle) {
      this.props.onContentViewToggle(contentView, this.state.value);
    }
  };

  /**
   * Component mounted into the DOM tree
   */
  componentDidMount() {
    const uaParser = new UAParser();
    if (uaParser.getOS().name && uaParser.getBrowser().name) {
      this.setState({
        deviceOsName: uaParser.getOS().name.toLowerCase(),
        deviceBrowserName: uaParser.getBrowser().name.toLowerCase(),
      });
    }
  }

  render() {
    let RichTextComponent = RichText;

    if (this.state.deviceOsName) {
      // Check if the user is using android. If so, then let's use RichTextIframe for now
      if (
        this.state.deviceOsName === 'android' ||
        this.state.deviceBrowserName === 'android browser'
      ) {
        RichTextComponent = RichTextIframe;
      }
    }

    return (
      <div className={this.props.classes.richTextContainer}>
        <EditorToolbar
          contentViewType={this.props.contentViewType}
          onStyleToggle={this._handleStyleToggle}
          onContentViewToggle={this._handleContentViewToggle}
        />
        <RichTextComponent
          commandToggleType={this.state.toggleType}
          commandToggleStyle={this.state.toggleStyle}
          onToggle={this._onToggle}
          onChange={this._onChange}
          onBlur={this._onBlur}
          onFocus={this._onFocus}
          value={this.state.value}
        />
      </div>
    );
  }
}

/**
 * Set accepted properties
 */
ContentHtml.propTypes = {
  /**
   * The callback function used when user changes the content of the editor
   *
   * @type {function}
   */
  onChange: PropTypes.func,

  /**
   * The callback function used when user looses the focus of the editor
   *
   * @type {function}
   */
  onBlur: PropTypes.func,

  /**
   * The initial value of the content editor
   *
   * @type {string}
   */
  value: PropTypes.string,

  /**
   * Handles the toggling of content view
   *
   * @type {function}
   */
  onContentViewToggle: PropTypes.func,
};

// Wrap in JSS for styles
export default injectStyles(styles, ContentHtml);
