import React, { Component } from 'react';
import PropTypes from 'prop-types';
import EditorToolbar from './EditorToolbar';
import { Controlled as CodeMirror } from 'react-codemirror2';
import { styles } from './styles';
import injectStyles from '../styles/injectStyles';

/**
 * Contains codemirror and the toolbar for viewing source
 */
class ContentSource extends Component {
  /**
   * Set accepted properties
   */
  static propTypes = {
    /**
     * The callback function used when user changes the content of the editor
     *
     * @type {function}
     */
    onChange: PropTypes.func,

    /**
     * The callback function used when user looses the focus of the editor
     *
     * @type {function}
     */
    onBlur: PropTypes.func,

    /**
     * The initial value of the content editor
     *
     * @type {string}
     */
    value: PropTypes.string,

    /**
     * Handles the toggling of content view
     *
     * @type {function}
     */
    onContentViewToggle: PropTypes.func,
  };

  /**
   * Class constructor
   *
   * @param {Object} props Properties to send to the render function
   */
  constructor(props) {
    // Call parent constructor
    super(props);

    this.state = {
      value: this.props.value,
      mountedInDom: false,
    };
  }

  /**
   * Handle when the editor has changed
   *
   * @param {Editor} editor The codemirror editor instnace
   * @param {Object} data Data about the change?
   * @param {string} value The value of the editor
   * @private
   */
  _onChange = (editor, data, value) => {
    if (this.props.onChange) {
      this.props.onChange(value);
    }

    this.setState({ value });
  };

  /**
   * Handles the toggling of content view
   *
   * @param {int} contentView The content view we are switching to
   * @private
   */
  _handleContentViewToggle = contentView => {
    if (this.props.onContentViewToggle) {
      this.props.onContentViewToggle(contentView, this.state.value);
    }
  };

  /**
   * Update the value if changed
   *
   * @param {Object} props
   */
  componentWillReceiveProps(props) {
    if (this.state.value != props.value) {
      this.setState({ value: props.value });
    }
  }

  render() {
    let options = {
      lineNumbers: true,
      mode: 'htmlmixed',
    };

    return (
      <div className={this.props.richTextContainer}>
        <EditorToolbar
          contentViewType={this.props.contentViewType}
          onContentViewToggle={this._handleContentViewToggle}
        />
        <CodeMirror value={this.state.value} onBeforeChange={this._onChange} options={options} />
      </div>
    );
  }
}

// Wrap in JSS for styles
export default injectStyles(styles, ContentSource);
