import React, { Component } from 'react';
import PropTypes from 'prop-types';
import { convertToHTML, convertFromHTML } from 'draft-convert';
import { Editor, EditorState, RichUtils } from 'draft-js';
import { styles } from './styles';
import injectStyles from '../styles/injectStyles';

// Custom overrides for "code" style.
const styleMap = {
  CODE: {
    backgroundColor: 'rgba(0, 0, 0, 0.05)',
    fontFamily: '"Inconsolata", "Menlo", "Consolas", monospace',
    fontSize: 16,
    padding: 2,
  },
};

/**
 * Chamel Editor for editing rich text and source code
 */
class RichText extends Component {
  /**
   * Set accepted properties
   */
  static propTypes = {
    /**
     * The callback function used when user changes the content of the editor
     *
     * @type {function}
     */
    onChange: PropTypes.func,

    /**
     * The callback function used when user toggles an icon of the editor
     *
     * @type {function}
     */
    onToggle: PropTypes.func,

    /**
     * The callback function used when user looses the focus of the editor
     *
     * @type {function}
     */
    onBlur: PropTypes.func,

    /**
     * The initial value of the content editor
     *
     * @type {string}
     */
    value: PropTypes.string,

    /**
     * The command that will be executed based on the type that was provided
     *
     * @type {string}
     */
    commandToggleType: PropTypes.string,

    /**
     * This will determine what style to toggle
     *
     * @type {string}
     */
    commandToggleStyle: PropTypes.oneOf(['block', 'inline']),
  };

  /**
   * Class constructor
   *
   * @param {Object} props Properties to send to the render function
   */
  constructor(props) {
    // Call parent constructor
    super(props);

    let initialEditorState = EditorState.createEmpty();

    this.refEditor = React.createRef();

    this.state = {
      editorState: initialEditorState,
    };
  }

  /**
   * Handle when the editor has changed
   *
   * @param {obj} editorState  The top-level state object for the editor.
   * @private
   */
  _onChange = editorState => {
    this.setState({ editorState });

    if (this.props.onChange) {
      //let content = stateToHTML(editorState.getCurrentContent());
      const content = convertToHTML(editorState.getCurrentContent());
      this.props.onChange(content);
    }
  };

  /**
   * Handles when the user set the focus in the editor
   *
   * @private
   */
  _focus = () => {
    if (this.refEditor) {
      this.refEditor.current.focus();
    }
    //this.refEditor.focus();

    if (this.props.onFocus) {
      //let content = stateToHTML(this.state.editorState.getCurrentContent());
      const content = convertToHTML(this.state.editorState.getCurrentContent());
      this.props.onFocus(content);
    }
  };

  /**
   * Handle when the editor looses the focus
   *
   * @private
   */
  _onBlur = () => {
    if (this.props.onBlur) {
      //let content = stateToHTML(this.state.editorState.getCurrentContent());
      const content = convertToHTML(this.state.editorState.getCurrentContent());
      this.props.onBlur(content);
    }
  };

  /**
   * Handle a RETURN keydown event
   *
   * @param {string} command The command that was sent
   * @returns {boolean}
   * @private
   */
  _handleKeyCommand = command => {
    const { editorState } = this.state;
    const newState = RichUtils.handleKeyCommand(editorState, command);

    if (newState) {
      this._onChange(newState);
      return true;
    }

    return false;
  };

  /**
   * Handle when user hits the tab button
   *
   * @param {DOMEvent} e Reference to the DOM event being sent
   * @returns {boolean}
   * @private
   */
  _onTab = e => {
    const maxDepth = 4;
    this._onChange(RichUtils.onTab(e, this.state.editorState, maxDepth));
  };

  /**
   * Handles the toggling of block types in the toolbar icons
   *
   * @param {string} blockType The block type that was clicked
   * @private
   */
  _toggleBlockType = blockType => {
    let editorState = RichUtils.toggleBlockType(this.state.editorState, blockType);

    if (this.props.onToggle) {
      //let content = stateToHTML(editorState.getCurrentContent());
      const content = convertToHTML(editorState.getCurrentContent());
      this.props.onToggle(content);
    }

    this.setState({ editorState });
  };

  /**
   * Handles the toggling of inline styles in the toolbar icons
   *
   * @param {string} inlineStyle The inline style that was clicked
   * @private
   */
  _toggleInlineStyle = inlineStyle => {
    let editorState = RichUtils.toggleInlineStyle(this.state.editorState, inlineStyle);

    if (this.props.onToggle) {
      //let content = stateToHTML(editorState.getCurrentContent());
      const content = convertToHTML(editorState.getCurrentContent());
      this.props.onToggle(content);
    }

    this.setState({ editorState });
  };

  /**
   * Handles the getting of block class name
   *
   * @param {obj} block Represents the state of the entire document.
   *
   * @return {string} Returns the current theme class name
   * @private
   */
  _getBlockStyle = block => {
    switch (block.getType()) {
      case 'blockquote':
        return this.props.classes.richTextBlockQuote;
      case 'unstyled':
        // Paragraph
        return this.props.classes.richTextDefaultBlock;
      default:
        return null;
    }
  };

  /**
   * Componenent is about to receive new props
   */
  componentWillReceiveProps(nextProps) {
    if (nextProps.commandToggleStyle === 'block') {
      this._toggleBlockType(nextProps.commandToggleType);
    } else if (nextProps.commandToggleStyle === 'inline') {
      this._toggleInlineStyle(nextProps.commandToggleType);
    }
  }

  componentDidMount() {
    // We wait until the editor mounts to set the value because it uses document to render
    if (this.props.value) {
      this.setState({
        editorState: EditorState.createWithContent(convertFromHTML(this.props.value)),
      });
    }
  }

  render() {
    const { editorState } = this.state;
    const propsClasses = this.props.classes;

    // Set style classes
    let classes = propsClasses.richText;

    // Determine if the editor has text
    let contentState = editorState.getCurrentContent();
    if (!contentState.hasText()) {
      if (
        contentState
          .getBlockMap()
          .first()
          .getType() !== 'unstyled'
      ) {
        classes += propsClasses.RichTextHidePlaceholder;
      }
    }

    // Append the className specified in the this.props
    if (this.props.className) {
      classes += ' ' + this.props.className;
    }

    let selection = editorState.getSelection();
    let blockType = editorState
      .getCurrentContent()
      .getBlockForKey(selection.getStartKey())
      .getType();

    return (
      <div className={propsClasses.richTextContainer}>
        <div className={classes} onClick={this._focus}>
          <Editor
            blockStyleFn={this._getBlockStyle}
            customStyleMap={styleMap}
            editorState={editorState}
            handleKeyCommand={this._handleKeyCommand}
            onChange={this._onChange}
            onBlur={this._onBlur}
            onTab={this._onTab}
            ref={this.refEditor}
            spellCheck={true}
          />
        </div>
      </div>
    );
  }
}

// Wrap in JSS for styles
export default injectStyles(styles, RichText);
