import React from 'react';
import PropTypes from 'prop-types';
import TouchRipple from '../ripples/TouchRipple';
import classnames from 'classnames';
import Checkbox from '../Toggle/Checkbox';
import { styles } from './styles';
import injectStyles from '../styles/injectStyles';
import BodyLeadText from '../Text/BodyLeadText';
import BodyText from '../Text/BodyText';

/**
 * Functional component for any button
 *
 * @param props
 * @returns {ReactDOM}
 * @constructor
 */
const ListItem = props => {
  const classes = classnames(props.classes.listItem, {
    [props.classes.listItemSelected]: props.selected,
    [props.classes.listItemEmphasized]: props.emphasized,
    [props.classes.listItemTappable]: props.onTap || props.onClick,
  });

  const contentClasses = classnames(props.classes.listItemContent, {
    [props.classes.listItemContentAlignTop]: props.children && !props.primaryText,
  });

  // Set ontap functions
  const actionCallback = props.onTap || props.onClick;
  const onTap = actionCallback ? actionCallback : () => {};
  let onLeftElementTap = null;

  if (props.onLeftElementTap) {
    onLeftElementTap = e => {
      e.stopPropagation();
      props.onLeftElementTap(e);
    };
  }

  // If we have a left element add it
  let leftElement = null;
  if (props.leftElement) {
    const leftIcon =
      props.selected && props.selectedShowCheckbox ? (
        <Checkbox checked={true} />
      ) : (
        props.leftElement
      );

    leftElement = (
      <div onClick={onLeftElementTap} className={props.classes.listItemLeft}>
        {leftIcon}
      </div>
    );
  }

  // If we have a right element add it
  const rightElement = props.rightElement ? (
    <div className={props.classes.listItemRight}>{props.rightElement}</div>
  ) : null;

  const bodyStyle = {
    margin: 0,
    overflow: 'hidden',
    textOverflow: 'ellipsis',
  };

  return (
    <div className={classes} onClick={onTap}>
      <TouchRipple disabled={props.onTap === null}>
        <div className={contentClasses}>
          {leftElement}
          <div className={props.classes.listItemData}>
            <div>
              <BodyLeadText style={bodyStyle}>{props.primaryText}</BodyLeadText>
              <BodyText style={bodyStyle}>{props.secondaryText}</BodyText>
            </div>
            {props.children}
          </div>
          {rightElement}
        </div>
      </TouchRipple>
    </div>
  );
};

/**
 * Set accepted properties
 */
ListItem.propTypes = {
  /**
   * Primary text/title of the item
   */
  primaryText: PropTypes.string,

  /**
   * Secondary text for the item - often a snippet of the body
   */
  secondaryText: PropTypes.string,

  /**
   * Optional children to render into the item
   */
  children: PropTypes.node,

  /**
   * Optional flag that can be set to indicate this item is selected
   */
  selected: PropTypes.bool,

  /**
   * Show a checkbox if this item is slected (or on mouse hover to indicate it can be)
   */
  selectedShowCheckbox: PropTypes.bool,

  /**
   * Used to highlight an entry - often used to mark a new entry
   */
  emphasized: PropTypes.bool,

  /**
   * Node element to render on the left side
   */
  leftElement: PropTypes.node,

  /**
   * Node element to render on the right side
   */
  rightElement: PropTypes.node,

  /**
   * Legacy event called when the primary action is tapped or Clicked
   */
  onTap: PropTypes.func,

  /**
   * Event called when the primary action is tapped or Clicked
   */
  onClick: PropTypes.func,

  /**
   * Event called when the left element is tapped or Clicked
   */
  onLeftElementTap: PropTypes.func,

  /**
   * JSS classes for styling the component
   */
  classes: PropTypes.object,
};

/**
 * Set property defaults
 */
ListItem.defaultProps = {
  primaryText: null,
  secondaryText: null,
  leftElement: null,
  rightElement: null,
  selected: false,
  emphasized: false,
  onTap: null,
};

export default injectStyles(styles, ListItem);
