import React, { Component } from 'react';
import PropTypes from 'prop-types';
import classnames from 'classnames';
import ArrowDropDownIcon from '../icons/svg/ArrowDropDownIcon';
import Menu from '../Menu/Menu';
import Popover from '../Popover/Popover';
import FontIcon from '../FontIcon';
import IconButton from '../Button/IconButton';
import { styles } from './styles';
import injectStyles from '../styles/injectStyles';

/**
 * Component for displaying dropdowns
 */
class SelectButton extends Component {
  /**
   * Class constructor takes properties and passes them to the parent/super
   */
  constructor(props) {
    super(props);

    // Popover anchor element
    this.popoverAncherEl = React.createRef();

    this.state = {
      open: false,
      selectedIndex: props.selectedIndex || 0,
    };
  }

  /**
   * Popover has entered the dom
   */
  componentDidMount() {
    if (this.props.hasOwnProperty('selectedIndex')) {
      this._setSelectedIndex(this.props);
    }
  }

  /**
   * Componenent is about to exit the dom
   */
  componentWillReceiveProps(nextProps) {
    if (this.props.hasOwnProperty('selectedIndex')) {
      this._setSelectedIndex(nextProps);
    }
  }

  /**
   * Render Componenent
   */
  render() {
    // Use default dropdown icon
    let icon = <ArrowDropDownIcon />;

    if (this.props.icon) {
      icon =
        typeof this.props.icon === 'string' ? (
          <FontIcon className={this.props.icon} />
        ) : (
          this.props.icon
        );
    }

    const className = classnames(this.props.classes.selectButtonCon, this.props.className);

    return (
      <div className={className}>
        <div style={{ align: 'top' }} ref={this.popoverAncherEl}>
          <IconButton className={this.props.className} onTap={this._onControlClick}>
            {icon}
          </IconButton>
        </div>
        <Popover
          open={this.state.open}
          anchorEl={this.popoverAncherEl}
          anchorOrigin={{ horizontal: 'left', vertical: 'top' }}
          onRequestClose={this._handleRequestClose}
        >
          <Menu
            //ref="menuItems"
            customClasses={this.props.classes[this.props.colorScheme]}
            autoWidth={this.props.autoWidth}
            selectedIndex={this.state.selectedIndex}
            menuItems={this.props.menuItems}
            onItemClick={this._onMenuItemClick}
          >
            {this.props.children}
          </Menu>
        </Popover>
      </div>
    );
  }

  /**
   * Set which menu item is selected
   *
   * @private
   * @param {Object} props The props we are setting
   */
  _setSelectedIndex = props => {
    const selectedIndex = props.selectedIndex;

    if (process.env.NODE_ENV !== 'production' && selectedIndex < 0) {
      console.warn('Cannot set selectedIndex to a negative index.', selectedIndex);
    }

    this.setState({ selectedIndex: selectedIndex > -1 ? selectedIndex : 0 });
  };

  /**
   * Meny control clicked handler
   *
   * @private
   * @param {DOMEvent} e The click event fired
   */
  _onControlClick = e => {
    e.preventDefault();

    this.setState({
      open: !this.state.open,
    });
  };

  /**
   * Triggered when a menu item gets clicked
   *
   * @private
   * @param {DOMEvent} e The event fired through
   * @param {int} key The index of the item clicked - this will be deprecated soon
   * @param {Object} payload Whatever payload was passed to the menu
   */
  _onMenuItemClick = (e, key, payload) => {
    if (this.props.onChange) {
      this.props.onChange(e, key, payload);
    }

    this.setState({
      selectedIndex: key,
      open: false,
    });

    // Prevent ghost clicks
    e.preventDefault();
    e.stopPropagation();

    // TODO: Not sure if this is needed with the above being called
    e.nativeEvent.stopImmediatePropagation();
  };

  /**
   * Handle when the popover gets closed
   *
   * @private
   * @param {DOMEvent} e The click event fired
   */
  _handleRequestClose = e => {
    this.setState({
      open: false,
    });
  };
}

/**
 * Set accepted properties
 */
SelectButton.propTypes = {
  onChange: PropTypes.func,
  selectedIndex: PropTypes.number,
  icon: PropTypes.oneOfType([PropTypes.string, PropTypes.element]),
  menuItems: PropTypes.array.isRequired,

  /**
   * Optional additional class for the icon button
   */
  className: PropTypes.string,

  /**
   * Optional children can be used to define content
   */
  children: PropTypes.node,

  /**
   * Optional color scheme to use
   *
   * dark = darker text color
   * light = light text color
   */
  txtColorScheme: PropTypes.oneOf(['primary', 'dark', 'light']),
};

/**
 * Set property defaults
 */
SelectButton.defaultProps = {
  autoWidth: true,
  children: null,
  className: null,
  colorScheme: 'light',
};

// Wrap in JSS for styles
export default injectStyles(styles, SelectButton);
