import React, { Component, Element } from 'react';
import PropTypes from 'prop-types';
import Events from '../utils/Events';
import Dom from '../utils/Dom';
import injectStyles from '../styles/injectStyles';
import { styles } from './styles';

/**
 * Main popover class handles absolute positioning paper relative to an element
 */
class Popover extends Component {
  /**
   * Class constructor
   *
   * @param {Object} props Properties to send to the render function
   */
  constructor(props) {
    // Call paprent constructor
    super(props);

    this.containingDiv = React.createRef();
  }

  /**
   * Set property defaults
   */
  static defaultProps = {
    open: false,
    anchorEl: null,
    zDepth: 1,
    relative: false,
    anchorOrigin: {
      vertical: 'bottom',
      horizontal: 'left',
    },
    targetOrigin: {
      vertical: 'top',
      horizontal: 'left',
    },
    pushToLeft: false,
  };

  /**
   * Set accepted properties
   */
  static propTypes = {
    open: PropTypes.bool,

    /**
     * Reference to element we are anchored to
     *
     * This should be a value created with React.createRef()
     */
    anchorEl: PropTypes.oneOfType([
      PropTypes.func,
      PropTypes.shape({ current: PropTypes.instanceOf(Element) }),
    ]).isRequired,

    /**
     * The depth of the float
     */
    zDepth: PropTypes.number,

    /**
     * JSS classes
     */
    classes: PropTypes.object,

    /**
     * If true, then position elements relative to parent and not document
     */
    relative: PropTypes.bool,

    /**
     * This is the point on the anchor where the popover
     * targetOrigin will stick to.
     * Options:
     * vertical: [top, middle, bottom]
     * horizontal: [left, center, right]
     */
    anchorOrigin: PropTypes.shape({
      vertical: PropTypes.oneOf(['top', 'middle', 'bottom']),
      horizontal: PropTypes.oneOf(['left', 'center', 'right']),
    }),

    /**
     * This is the point on the popover which will stick to
     * the anchors origin.
     * Options:
     * vertical: [top, middle, bottom]
     * horizontal: [left, center, right]
     */
    targetOrigin: PropTypes.shape({
      vertical: PropTypes.oneOf(['top', 'middle', 'bottom']),
      horizontal: PropTypes.oneOf(['left', 'center', 'right']),
    }),

    /**
     * If true, then position elements will push to left to show the
     * popover without covering the trigger button.
     */
    pushToLeft: PropTypes.bool,
  };

  /**
   * Popover has entered the dom
   */
  componentDidMount() {
    Events.on(document, 'click', this._checkClickAway);
  }

  /**
   * Componenent is about to exit the dom
   */
  componentWillUnmount() {
    Events.off(document, 'click', this._checkClickAway);
  }

  /**
   * Triggered after the component receives updated props
   */
  componentDidUpdate() {
    this._setPlacement();
  }

  /**
   * Render the popover
   */
  render() {
    let classes = this.props.classes.popover;
    if (this.props.open) {
      classes += ' ' + this.props.classes.popoverVisible;
      if (this.props.pushToLeft) {
        classes += ' ' + this.props.classes.pushToLeft;
      }
    }

    if (this.props.children && this.props.children.props.hasOwnProperty('menuItems')) {
      if (this.props.children.props.menuItems.length > 12) {
        classes += ' ' + this.props.classes.addScrollBar;
      }
    }

    return (
      <div ref={this.containingDiv} className={classes}>
        {this.props.children}
      </div>
    );
  }

  /**
   * Handle when the user clicks away from the popup
   */
  _checkClickAway = e => {
    // Wrap in try in case the DOM element is no longer mounted
    try {
      // Make sure that the elements are still in the dom
      const el = this.containingDiv.current;
      const anchorEl = this.props.anchorEl.current;

      // Check if the target is inside the current component
      if (
        this.props.open &&
        e.target != el &&
        !Dom.isDescendant(el, e.target) &&
        e.target != anchorEl &&
        !Dom.isDescendant(anchorEl, e.target) &&
        document.documentElement.contains(e.target)
      ) {
        if (this.props.onRequestClose) {
          this.props.onRequestClose();
        }
      }
    } catch (error) {
      // DOM node is no longer in the document (aka, it's closed already)
      if (this.props.onRequestClose) {
        this.props.onRequestClose();
      }
    }
  };

  /**
   * Reposition the popover on the page
   */
  _setPlacement() {
    if (!this.props.open || typeof document == 'undefined') {
      return;
    }

    const targetEl = this.containingDiv.current;
    const anchorEl = this.props.anchorEl.current;

    if (anchorEl === null) {
      console.error('Huh', this.props.anchorEl);
      throw 'anchorEl has no current';
    }

    const { targetOrigin, anchorOrigin } = this.props;

    let anchorPosition = Dom.offset(anchorEl);

    // If we are in a relative positioned container, then set top and left to 0
    targetEl.style.position = 'absolute';
    if (this.props.relative) {
      targetEl.style.position = 'relative';
    }
    if (targetOrigin.vertical == 'top') {
      anchorPosition.top = 0;
    }
    if (targetOrigin.horizontal == 'left') {
      anchorPosition.left = 0;
    }

    /*
     * Determine relative positions based on the anchor element coords
     */
    const relativeAnchorPosition = {
      top: anchorPosition.top,
      middle: anchorPosition.top + anchorPosition.height / 2,
      bottom: anchorPosition.top + anchorPosition.height,
      left: anchorPosition.left,
      center: anchorPosition.left + anchorPosition.width / 2,
      right: anchorPosition.left + anchorPosition.width,
    };

    let targetPosition = {
      top: relativeAnchorPosition[anchorOrigin.vertical],
      left: relativeAnchorPosition[anchorOrigin.horizontal],
    };

    targetEl.style.top = `${Math.max(0, targetPosition.top)}px`;
    targetEl.style.left = `${Math.max(0, targetPosition.left)}px`;
    targetEl.style.maxHeight = `${window.innerHeight}px`;
    targetEl.style.maxWidth = `${window.innerWidth}px`;
    targetEl.style.width = 'auto';

    // Update position if out of viewing bounds
    this._applyAutoPositionIfNeeded(targetPosition, targetEl);
  }

  /**
   * If needed we can reposition based on current viewport bounds
   *
   * Make sure that the popover is not off the viewport to the right or bottom
   * of the page.
   *
   * @private
   * @param {Object} relativeTargetPosition The current relative top and left
   *  props relative to the anchorElement.
   * @param {DOMElement} targetEl Floating popover DOM element being repositioned
   */
  _applyAutoPositionIfNeeded(relativeTargetPosition, targetEl) {
    const targetPosition = Dom.offset(targetEl);

    /*
     * We need to get the scroll top so we can offset it with the target position top
     * After getting the offset top, we can now properly compute for the proper top position value
     */
    const doc = document.documentElement;
    const scrollTop = window.pageYOffset || doc.scrollTop;
    const offsetTop = targetPosition.top - scrollTop;

    // Move target position just to the top if the display will be out of bounds
    if (offsetTop < 0) {
      // Apply the new position
      const archorPos = Dom.offset(this.props.anchorEl);
      targetEl.style.top = `${Math.abs(archorPos.top)}px`;
    }

    // Movethe target position up so it is not scrolling past the bottom
    if (offsetTop + targetPosition.height > window.innerHeight) {
      // Initialize new top position
      let newTop = relativeTargetPosition.top;

      // Subtract enough pixels to get it inside the bounds of the window
      newTop -= offsetTop + targetPosition.height - window.innerHeight;

      // Apply the new position
      targetEl.style.top = `${newTop}px`;
    }

    // Move target position just to the left of the right outer bounds
    if (targetPosition.right > window.innerWidth) {
      // Initialize new left position
      let newLeft = relativeTargetPosition.left;

      // Subtract enough pixels to get it inside the bounds of the window
      newLeft -= targetPosition.right - window.innerWidth;

      // Apply the new position
      targetEl.style.left = `${newLeft}px`;
    }
  }
}

export default injectStyles(styles, Popover);
