import React, { Component } from 'react';
import PropTypes from 'prop-types';
import prefixer from '../utils/prefixer.js';
import classnames from 'classnames';
import injectStyles from '../styles/injectStyles';
import { styles } from './styles';

class Progress extends Component {
  static propTypes = {
    buffer: PropTypes.number,
    className: PropTypes.string,
    disabled: PropTypes.bool,
    max: PropTypes.number,
    min: PropTypes.number,
    mode: PropTypes.oneOf(['determinate', 'indeterminate']),
    multicolor: PropTypes.bool,
    type: PropTypes.oneOf(['linear', 'circular']),
    value: PropTypes.number,
    size: PropTypes.oneOf(['small', 'normal']),
  };

  static defaultProps = {
    buffer: 0,
    className: '',
    max: 100,
    min: 0,
    mode: 'indeterminate',
    multicolor: false,
    type: 'circular',
    value: 0,
    size: 'normal',
  };

  /**
   * Class constructor
   *
   * @param {Object} props Properties to send to the render function
   */
  constructor(props) {
    // Call paprent constructor
    super(props);
  }

  calculateRatio(value) {
    if (value < this.props.min) return 0;
    if (value > this.props.max) return 1;
    return (value - this.props.min) / (this.props.max - this.props.min);
  }

  circularStyle() {
    if (this.props.mode !== 'indeterminate') {
      return {
        strokeDasharray: `${2 * Math.PI * 25 * this.calculateRatio(this.props.value)}, 400`,
      };
    }
  }

  linearStyle() {
    if (this.props.mode !== 'indeterminate') {
      return {
        buffer: prefixer({ transform: `scaleX(${this.calculateRatio(this.props.buffer)})` }),
        value: prefixer({ transform: `scaleX(${this.calculateRatio(this.props.value)})` }),
      };
    } else {
      return {};
    }
  }

  renderCircular() {
    const { classes, mode, multicolor } = this.props;

    const circleClassNames = classnames(classes.circle, { [classes[mode + 'Circle']]: mode });

    const pathClassNames = classnames(
      classes.path,
      { [classes[mode + 'Path']]: mode },
      { [classes[mode + 'MulticolorPath']]: multicolor },
    );

    return (
      <svg className={circleClassNames} viewBox="0 0 60 60">
        <circle className={pathClassNames} style={this.circularStyle()} cx="30" cy="30" r="25" />
      </svg>
    );
  }

  renderLinear() {
    const { buffer, value } = this.linearStyle();

    return (
      <div>
        <span data-ref="buffer" className={this.props.classes.buffer} style={buffer} />
        <span data-ref="value" className={this.getClassNamesForLinearValue()} style={value} />
      </div>
    );
  }

  /**
   * Function that will create class names for linear value
   */
  getClassNamesForLinearValue() {
    const { classes, mode } = this.props;

    return classnames(classes.value, { [classes[mode + 'Value']]: mode });
  }

  render() {
    const { className, classes, disabled, max, min, mode, type, value, size } = this.props;

    const _className = classnames(
      classes[type],
      { [classes[mode]]: mode },
      classes[type + size + 'Size'],
    );

    return (
      <div
        disabled={disabled}
        data-react-toolbox="progress-bar"
        aria-valuenow={value}
        aria-valuemin={min}
        aria-valuemax={max}
        className={_className}
      >
        {type === 'circular' ? this.renderCircular() : this.renderLinear()}
      </div>
    );
  }
}

// Wrap in JSS for styles
export default injectStyles(styles, Progress);
