import React from 'react';
import PropTypes from 'prop-types';
import { styles } from './styles';
import injectStyles from '../styles/injectStyles';
import classnames from 'classnames';
import Checkbox from '../Toggle/Checkbox';

class Row extends React.Component {
  /**
   * Set some sane defaults
   */
  static defaultProps = {
    showCheckBox: true,
    onLeftElementTap: null,
  };

  static propTypes = {
    /**
     * Contents of the cell
     */
    children: PropTypes.any,

    /**
     * JSS classes for styling the component
     */
    classes: PropTypes.object,

    /**
     * Optional flag that can be set to indicate this item is selected
     */
    selected: PropTypes.bool,

    /**
     * Used to highlight an entry - often used to mark a new entry
     */
    emphasized: PropTypes.bool,

    /**
     * Event called when the primary action is tapped or Clicked
     */
    onTap: PropTypes.func,

    /**
     * If set, then unselected rows will display the icon element
     */
    iconElement: PropTypes.node,

    /**
     * If true then each row adds a column for a checkbox
     */
    showCheckBox: PropTypes.bool,

    /**
     * Callback called any time the user taps the checkbox or icon
     */
    onLeftElementTap: PropTypes.func,
  };

  /**
   * Class constructor
   *
   * @param {Object} props Properties to send to the render function
   */
  constructor(props) {
    super(props);

    this.state = {
      mouseIsHoveredOver: false,
    };
  }

  render() {
    const classNames = classnames(this.props.classes.row, {
      [this.props.classes.rowEmphasized]: this.props.emphasized,
      [this.props.classes.rowSelected]: this.props.selected,
      [this.props.classes.rowTappable]: this.props.onTap,
    });

    // Find out if we should display a checkbox column
    let checkBoxColumn = null;
    if (this.props.showCheckBox) {
      let showElement = <Checkbox checked={this.props.selected} />;

      // If iconElement is not null and item is not selected, then use icon
      if (this.props.iconElement && !this.props.selected && !this.state.mouseIsHoveredOver) {
        showElement = this.props.iconElement;
      }
      checkBoxColumn = (
        <td
          className={this.props.classes.cell + ' ' + this.props.classes.iconCell}
          onClick={event => {
            event.preventDefault(); // Let's stop this event.
            event.stopPropagation(); // Really this time.
            this.props.onLeftElementTap();
          }}
        >
          {showElement}
        </td>
      );
    }

    return (
      <tr
        onClick={this.props.onTap}
        className={classNames}
        onMouseEnter={() => {
          this.setState({ mouseIsHoveredOver: true });
        }}
        onMouseLeave={() => {
          this.setState({ mouseIsHoveredOver: false });
        }}
      >
        {checkBoxColumn}
        {this.props.children}
      </tr>
    );
  }
}

export default injectStyles(styles, Row);
