import React, { Component } from 'react';
import PropTypes from 'prop-types';
import { ThemeProvider } from 'react-jss';
import jss from 'jss';
import preset from 'jss-preset-default';
import reboot from './reboot';
import { getTheme } from './themeAutoSelector';
import { styles as globalStyles } from './globalStyles';

// Use default plugins - https://github.com/cssinjs/jss-preset-default
jss.setup(preset());

/* 
 * Add reboot.css. Set render index to -100002 because react-jss starts
 * injecting at -100000 in order to make sure components are rendered
 * in the correct order right before the theme global styles @ -1000001
 * See https://github.com/cssinjs/react-jss/blob/master/src/injectSheet.js
 */
jss.createStyleSheet(reboot, { index: -100002, meta: 'reboot' }).attach();

/**
 * Theme provider to provide chameleon styles to all children
 */
class ChamelThemeProvider extends Component {
  static propTypes = {
    /**
     * All the descendent children of this provider
     */
    children: PropTypes.element,

    /**
     * Theme properties object
     */
    theme: PropTypes.object,

    /**
     * If true the theme will set global document styles
     */
    setGlobalStyles: PropTypes.bool,
  };

  /**
   * We provide the selected theme for any child components to access directly
   */
  static childContextTypes = {
    chamelTheme: PropTypes.object,
  };

  /**
   * Pass theme down for any child components to access directly
   */
  getChildContext() {
    return {
      chamelTheme: this.state.selectedTheme,
    };
  }

  /**
   * Class constructor takes properties and passes them to the parent/super
   *
   * @param {Object} props React props
   */
  constructor(props) {
    super(props);

    // Reference to the global style sheet if set
    this.globalStyleSheet = null;

    this.state = {
      selectedTheme: props.theme || getTheme(),
    };
  }

  /**
   * Set global settings on mount
   */
  componentDidMount() {
    if (this.state.selectedTheme && this.props.setGlobalStyles) {
      this.setGlobalStyles();
    }
  }

  /**
   * If no theme was provided on mount, then retrieve from user agent
   */
  componentDidUpdate(prevProps) {
    if (this.props.theme != prevProps.theme && this.props.theme != this.state.selectedTheme) {
      this.setState({
        selectedTheme: this.props.theme || getTheme(),
      });
      this.setGlobalStyles();
    }
  }

  /* 
   * Add global styles from the theme settings.
   * injecting at -1000002 in order to make sure components are rendered
   * in the correct order right after reboot @ -1000001
   * See https://github.com/cssinjs/react-jss/blob/master/src/injectSheet.js
   */
  setGlobalStyles() {
    if (this.globalStyleSheet) {
      jss.removeStyleSheet(this.globalStyleSheet);
    }

    // Create the style sheet
    this.globalStyleSheet = jss.createStyleSheet(globalStyles(this.state.selectedTheme), {
      index: -100002,
      meta: 'themeglobal',
    });

    this.globalStyleSheet.attach();
  }

  /**
   * Render into the DOM
   */
  render() {
    const theme = this.state.selectedTheme ? this.state.selectedTheme : {};
    return <ThemeProvider theme={theme}>{this.props.children}</ThemeProvider>;
  }
}

export default ChamelThemeProvider;
