/**
 * Convert boostrap4's reboot.css into JSS
 * https://github.com/twbs/bootstrap/blob/v4-dev/dist/css/bootstrap-reboot.css
 */
const reboot = {
  '@global': {
    /*
    * Reboot box sizing so that elements 
    * are never larger than width regardless of padding
    */
    '*::before, *::after': {
      boxSizing: 'border-box',
    },

    /*
     * Set some global initial font values 
     */
    html: {
      fontFamily: 'sans-serif',
      lineHeight: '1.15',
      webkitTextSizeAdjust: '100%',
      msTextSizeAdjust: '100%',
      msOverflowStyle: 'scrollbar',
      webkitTapHighlightColor: 'transparent',
    },

    /*
     * IE10 had an issue with auto-width
     */
    '@-ms-viewport': {
      width: 'device-width',
    },

    /*
     * Set new html5 block level elements to display block 
     */
    'article, aside, dialog, figcaption, figure, footer, header, hgroup, main, nav, section': {
      display: 'block',
    },

    /*
     * Global body settings with native (OS specific) fonts
     */
    body: {
      margin: '0',
      fontFamily:
        '-apple-system, BlinkMacSystemFont, "Segoe UI", Roboto, "Helvetica Neue", Arial, sans-serif, "Apple Color Emoji", "Segoe UI Emoji", "Segoe UI Symbol"',
      fontSize: '1rem',
      fontWeight: '400',
      lineHeight: '1.5',
      color: '#212529',
      textAlign: 'left',
      backgroundColor: '#fff',
    },

    // Suppress the focus outline on elements that cannot be accessed via keyboard.
    // This prevents an unwanted focus outline from appearing around elements that
    // might still respond to pointer events.
    //
    // Credit: https://github.com/suitcss/base
    '[tabindex="-1"]:focus': {
      outline: '0 !important',
    },

    // Content grouping
    //
    // 1. Add the correct box sizing in Firefox.
    // 2. Show the overflow in Edge and IE.
    hr: {
      boxSizing: 'content-box',
      height: '0',
      overflow: 'visible',
    },

    //
    // Typography
    //

    // Remove top margins from headings
    //
    // By default, `<h1>`-`<h6>` all receive top and bottom margins. We nuke the top
    // margin for easier control within type scales as it avoids margin collapsing.
    // stylelint-disable selector-list-comma-newline-after
    'h1, h2, h3, h4, h5, h6': {
      marginTop: '0',
      marginBottom: '0.5rem',
    },

    // Reset margins on paragraphs
    //
    // Similarly, the top margin on `<p>`s get reset. However, we also reset the
    // bottom margin to use `rem` units instead of `em`.
    p: {
      marginTop: '0',
      marginBottom: '1rem',
    },

    // Abbreviations
    //
    // 1. Remove the bottom border in Firefox 39-.
    // 2. Add the correct text decoration in Chrome, Edge, IE, Opera, and Safari.
    // 3. Add explicit cursor to indicate changed behavior.
    // 4. Duplicate behavior to the data-* attribute for our tooltip plugin
    'abbr[title], abbr[data-original-title]': {
      textDecoration: 'underline dotted', // 2
      webkitTextDecoration: 'underline dotted', // 2
      fallbacks: [
        {
          textDecoration: 'underline', //2
        },
      ],
      cursor: 'help', // 3
      borderBottom: '0', // 1
    },
    address: {
      marginBottom: '1rem',
      fontStyle: 'normal',
      lineHeight: 'inherit',
    },
    'ol, ul, dl': {
      marginTop: '0',
      marginBottom: '1rem',
    },
    'ol ol, ul ul, ol ul, ul ol': {
      marginBottom: '0',
    },
    dt: {
      fontWeight: '700',
    },
    dd: {
      marginBottom: '.5rem',
      // Undo browser default
      marginLeft: '0',
    },
    blockquote: {
      margin: '0 0 1rem',
    },
    dfn: {
      // Add the correct font style in Android 4.3-
      fontStyle: 'italic',
    },

    // stylelint-disable font-weight-notation
    'b, strong': {
      // Add the correct font weight in Chrome, Edge, and Safari
      fontWeight: 'bolder',
    },
    small: {
      fontSize: '80%',
    },
    'sub, sup': {
      position: 'relative',
      fontSize: '75%',
      lineHeight: '0',
      verticalAlign: 'baseline',
    },
    sub: {
      bottom: '-.25em',
      top: '-.5em',
    },

    //
    // Links
    //

    a: {
      color: '#007bff',
      textDecoration: 'none',
      // Remove the gray background on active links in IE 10.
      backgroundColor: 'transparent',
      // Remove gaps in links underline in iOS 8+ and Safari 8+.
      webkitTextDecorationSkip: 'objects',
    },
    'a:hover': {
      color: '#0056b3',
      textDecoration: 'underline',
    },

    // And undo these styles for placeholder links/named anchors (without href)
    // which have not been made explicitly keyboard-focusable (without tabindex).
    // It would be more straightforward to just use a[href] in previous block, but that
    // causes specificity issues in many other styles that are too complex to fix.
    // See https://github.com/twbs/bootstrap/issues/19402
    'a:not([href]):not([tabindex])': {
      color: 'inherit',
      textDecoration: 'none',
    },
    'a:not([href]):not([tabindex]):hover, a:not([href]):not([tabindex]):focus': {
      color: 'inherit',
      textDecoration: 'none',
    },
    'a:not([href]):not([tabindex]):focus': {
      outline: '0',
    },

    //
    // Code
    //

    // stylelint-disable font-family-no-duplicate-names
    'pre, code, kbd, samp': {
      fontFamily: 'monospace, monospace',
      fontSize: '1em',
    },
    // stylelint-enable font-family-no-duplicate-names

    pre: {
      // Remove browser default top margin
      marginTop: '0',
      // Reset browser default of `1em` to use `rem`s
      marginBottom: '1rem',
      // Don't allow content to break outside
      overflow: 'auto',
      // We have @viewport set which causes scrollbars to overlap content in IE11 and Edge, so
      // we force a non-overlapping, non-auto-hiding scrollbar to counteract.
      msOverflowStyle: 'scrollbar',
    },

    //
    // Figures
    //

    figure: {
      // Apply a consistent margin strategy (matches our type styles).
      margin: '0 0 1rem',
    },

    //
    // Images and content
    //

    img: {
      verticalAlign: 'middle',
      // Remove the border on images inside links in IE 10-.
      borderStyle: 'none',
    },
    'svg:not(:root)': {
      // Hide the overflow in IE
      overflow: 'hidden',
    },

    //
    // Tables
    //

    table: {
      // Prevent double borders
      borderCollapse: 'collapse',
    },

    caption: {
      paddingTop: '0.75rem',
      paddingBottom: '0.75rem',
      color: '#6c757d',
      textAlign: 'left',
      captionSide: 'bottom',
    },
    th: {
      // Matches default `<td>` alignment by inheriting from the `<body>`, or the
      // closest parent with a set `text-align`.
      textAlign: 'inherit',
    },

    //
    // Forms
    //

    label: {
      // Allow labels to use `margin` for spacing.
      display: 'inline-block',
      marginBottom: '.5rem',
    },

    // Remove the default `border-radius` that macOS Chrome adds.
    //
    // Details at https://github.com/twbs/bootstrap/issues/24093
    button: {
      borderRadius: '0',
    },

    // Work around a Firefox/IE bug where the transparent `button` background
    // results in a loss of the default `button` focus styles.
    //
    // Credit: https://github.com/suitcss/base/
    'button:focus': {
      outline: '5px auto -webkit-focus-ring-color',
      fallbacks: [
        {
          outline: '1px dotted',
        },
      ],
    },
    'input, button, select, optgroup, textarea': {
      margin: '0',
      fontFamily: 'inherit',
      fontSize: 'inherit',
      lineHeight: 'inherit',
    },
    'button, input': {
      // Show the overflow in Edge
      overflow: 'visible',
    },
    'button, select': {
      // Remove the inheritance of text transform in Firefox
      textTransform: 'none',
    },

    // 1. Prevent a WebKit bug where (2) destroys native `audio` and `video`
    //    controls in Android 4.
    // 2. Correct the inability to style clickable types in iOS and Safari.
    'button, html [type="button"], [type="reset"], [type="submit"]': {
      webkitAppearance: 'button',
    },

    // Remove inner border and padding from Firefox, but don't restore the outline like Normalize.
    'button::-moz-focus-inner, [type="button"]::-moz-focus-inner, [type="reset"]::-moz-focus-inner, [type="submit"]::-moz-focus-inner': {
      padding: '0',
      borderStyle: 'none',
    },
    'input[type="radio"], input[type="checkbox"]': {
      // 1. Add the correct box sizing in IE 10-
      boxSizing: 'border-box',
      // 2. Remove the padding in IE 10-
      padding: '0',
    },
    'input[type="date"], input[type="time"], input[type="datetime-local"], input[type="month"]': {
      // Remove the default appearance of temporal inputs to avoid a Mobile Safari
      // bug where setting a custom line-height prevents text from being vertically
      // centered within the input.
      // See https://bugs.webkit.org/show_bug.cgi?id=139848
      // and https://github.com/twbs/bootstrap/issues/11266
      webkitAppearance: 'listbox',
    },
    textarea: {
      // Remove the margin in Firefox and Safari
      overflow: 'auto',
      // Textareas should really only resize vertically so they don't break their (horizontal) containers.
      resize: 'vertical',
    },
    fieldset: {
      // Browsers set a default `min-width: min-content;` on fieldsets,
      // unlike e.g. `<div>`s, which have `min-width: 0;` by default.
      // So we reset that to ensure fieldsets behave more like a standard block element.
      // See https://github.com/twbs/bootstrap/issues/12359
      // and https://html.spec.whatwg.org/multipage/#the-fieldset-and-legend-elements
      minWidth: '0',
      // Reset the default outline behavior of fieldsets so they don't affect page layout.
      padding: '0',
      margin: '0',
      border: '0',
    },

    // 1. Correct the text wrapping in Edge and IE.
    // 2. Correct the color inheritance from `fieldset` elements in IE.
    legend: {
      display: 'block',
      width: '100%',
      maxWidth: '100%',
      padding: '0',
      marginBottom: '.5rem',
      fontSize: '1.5rem',
      lineHeight: 'inherit',
      color: 'inherit',
      whiteSpace: 'normal',
    },

    progress: {
      // Add the correct vertical alignment in Chrome, Firefox, and Opera.
      verticalAlign: 'baseline',
    },

    // Correct the cursor style of increment and decrement buttons in Chrome.
    '[type="number"]::-webkit-inner-spin-button, [type="number"]::-webkit-outer-spin-button': {
      height: 'auto',
    },

    '[type="search"]': {
      // This overrides the extra rounded corners on search inputs in iOS so that our
      // `.form-control` class can properly style them. Note that this cannot simply
      // be added to `.form-control` as it's not specific enough. For details, see
      // https://github.com/twbs/bootstrap/issues/11586.
      outlineOffset: -2, // 2. Correct the outline style in Safari.
      webkitAppearance: 'none',
    },

    //
    // Remove the inner padding and cancel buttons in Chrome and Safari on macOS.
    //

    '[type="search"]::-webkit-search-cancel-button, [type="search"]::-webkit-search-decoration': {
      webkitAppearance: 'none',
    },

    //
    // 1. Correct the inability to style clickable types in iOS and Safari.
    // 2. Change font properties to `inherit` in Safari.
    //

    '::-webkit-file-upload-button': {
      font: 'inherit',
      webkitAppearance: 'button',
    },

    //
    // Correct element displays
    //

    output: {
      display: 'inline-block',
    },
    summary: {
      // Add the correct display in all browsers
      display: 'list-item',
      cursor: 'pointer',
    },
    template: {
      // Add the correct display in IE
      display: 'none',
    },

    // Always hide an element with the `hidden` HTML attribute (from PureCSS).
    // Needed for proper display in IE 10-.
    '[hidden]': {
      display: 'none !important',
    },
  },
};
export default reboot;
